﻿using ShutdownTimer.Core.Models;

using System;

namespace ShutdownTimer.Core.Helpers
{
    public static class Clockwork
    {
        private static Timer timer; // default timer which is static and shares the same data for all instances of Clockwork

        /// <summary>
        /// Check if timer is initialized.
        /// </summary>
        private static void CheckTimer()
        {
            if (timer == null)
            {
                timer = new Timer();
            }
        }

        /// <summary>
        /// Returns current state of the timer.
        /// </summary>
        /// <returns>Current values of timer</returns>
        public static Timer GetTimer()
        {
            CheckTimer();
            return timer;
        }

        /// <summary>
        /// Changes the value of a current timer to any value.
        /// Note: When you are using this to initialize a Timer, make sure to set initializeTime to true.
        /// </summary>
        /// <param name="hours">New hour value</param>
        /// <param name="minutes">New minute value</param>
        /// <param name="seconds">New second value</param>
        /// <param name="currentTime">Changes the current time.</param>
        /// <param name="initialTime">Changes the timestamp at which the timer started. Set this to true when initializing a Timer.</param>
        public static void UpdateTime(int hours, int minutes, int seconds, bool currentTime = true, bool initialTime = false)
        {
            CheckTimer();

            if (currentTime)
            {
                timer.hours = hours;
                timer.minutes = minutes;
                timer.seconds = seconds;
            }

            if (initialTime)
            {
                timer.initialHours = hours;
                timer.initialMinutes = minutes;
                timer.initialSeconds = seconds;
            }
        }

        /// <summary>
        /// Sets the action value of a timer.
        /// </summary>
        /// <param name="action">New action value</param>
        public static void SetAction(string action)
        {
            CheckTimer();

            timer.action = action;
        }

        /// <summary>
        /// Counts a timer down one second.
        /// Make sure to validate the Timer beforehand or this will crash!
        /// </summary>
        public static void CountDown()
        {
            CheckTimer();

            if (timer.hours < 0 || timer.minutes < 0 || timer.seconds < 0)
            {
                // something has gone wrong and atleast one of the counters is below zero
                throw new NotImplementedException();
            }

            if (timer.seconds > 0)
            {
                timer.seconds--;
            }
            else if (timer.minutes > 0)
            {
                timer.seconds = 59;
                timer.minutes--;
            }
            else if (timer.hours > 0)
            {
                timer.seconds = 59;
                timer.minutes = 59;
                timer.hours--;
            }
            else
            {
                // every counter is on zero but there was no check in place to prevent this method from running
                throw new NotImplementedException();
            }
        }

        /// <summary>
        /// Counts a timer up one second.
        /// Make sure to validate the Timer beforehand or this will crash!
        /// </summary>
        public static void CountUp()
        {
            CheckTimer();

            if (timer.minutes > 59 || timer.seconds > 59)
            {
                // something has gone wrong and minutes or seconds are over value of 59
                throw new NotImplementedException();
            }

            if (timer.seconds < 59)
            {
                timer.seconds++;
            }
            else if (timer.minutes < 59)
            {
                timer.seconds = 0;
                timer.minutes++;
            }
            else
            {
                timer.seconds = 0;
                timer.minutes = 0;
                timer.hours++;
            }
        }

        /// <summary>
        /// Checks if the timer has hit zero.
        /// </summary>
        /// <returns>Returns thruthy response</returns>
        public static bool IsZero()
        {
            if (timer.hours == 0 && timer.minutes == 0 && timer.seconds == 0)
            {
                return true;
            }
            else
            {
                return false;
            }
        }
    }
}
